@@ -1,12 +1,35 @@
 /*
- * MUSCLE SmartCard Development ( http://www.linuxnet.com )
+ * MUSCLE SmartCard Development ( http://pcsclite.alioth.debian.org/pcsclite.html )
  *
  * Copyright (C) 1999
- *  David Corcoran <corcoran@linuxnet.com>
- * Copyright (C) 2004-2008
+ *  David Corcoran <corcoran@musclecard.com>
+ * Copyright (C) 2004-2010
  *  Ludovic Rousseau <ludovic.rousseau@free.fr>
  *
- * $Id: testpcsc.c 4328 2009-07-20 12:11:18Z rousseau $
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
+
+1. Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+2. Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+3. The name of the author may not be used to endorse or promote products
+   derived from this software without specific prior written permission.
+
+THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Id: testpcsc.c 7004 2014-10-02 09:26:36Z rousseau $
  */
 
 /**
@@ -56,7 +79,7 @@ int main(/*@unused@*/ int argc, /*@unused@*/ char **argv)
 {
 	SCARDHANDLE hCard;
 	SCARDCONTEXT hContext;
-	SCARD_READERSTATE_A rgReaderStates[1];
+	SCARD_READERSTATE rgReaderStates[1];
 	DWORD dwReaderLen, dwState, dwProt, dwAtrLen;
 	DWORD dwPref, dwReaders = 0;
 	char *pcReaders = NULL, *mszReaders;
@@ -78,9 +101,9 @@ int main(/*@unused@*/ int argc, /*@unused@*/ char **argv)
 	long rv;
 	DWORD i;
 	int p, iReader;
-	int iList[16];
-	SCARD_IO_REQUEST pioRecvPci;
-	SCARD_IO_REQUEST pioSendPci;
+	int iList[16] = {0};
+	SCARD_IO_REQUEST ioRecvPci = *SCARD_PCI_T0;	/* use a default value */
+	const SCARD_IO_REQUEST *pioSendPci;
 	unsigned char bSendBuffer[MAX_BUFFER_SIZE];
 	unsigned char bRecvBuffer[MAX_BUFFER_SIZE];
 	DWORD send_length, length;
@@ -185,10 +208,14 @@ wait_for_card_again:
 		do
 		{
 			char input[80];
+			char *r;
 
 			printf("Enter the reader number\t\t: ");
-			(void)fgets(input, sizeof(input), stdin);
-			(void)sscanf(input, "%d", &iReader);
+			r = fgets(input, sizeof(input), stdin);
+			if (NULL == r)
+				iReader = -1;
+			else
+				iReader = atoi(input);
 
 			if (iReader > p || iReader <= 0)
 				printf("Invalid Value - try again\n");
@@ -219,13 +246,13 @@ wait_for_card_again:
 	switch(dwPref)
 	{
 		case SCARD_PROTOCOL_T0:
-			pioSendPci = *SCARD_PCI_T0;
+			pioSendPci = SCARD_PCI_T0;
 			break;
 		case SCARD_PROTOCOL_T1:
-			pioSendPci = *SCARD_PCI_T1;
+			pioSendPci = SCARD_PCI_T1;
 			break;
 		case SCARD_PROTOCOL_RAW:
-			pioSendPci = *SCARD_PCI_RAW;
+			pioSendPci = SCARD_PCI_RAW;
 			break;
 		default:
 			printf("Unknown protocol\n");
@@ -242,8 +269,8 @@ wait_for_card_again:
 	length = sizeof(bRecvBuffer);
 
 	printf("Testing SCardTransmit\t\t: ");
-	rv = SCardTransmit(hCard, &pioSendPci, bSendBuffer, send_length,
-		&pioRecvPci, bRecvBuffer, &length);
+	rv = SCardTransmit(hCard, pioSendPci, bSendBuffer, send_length,
+		&ioRecvPci, bRecvBuffer, &length);
 	test_rv(rv, hContext, PANIC);
 	printf(" card response:" GREEN);
 	for (i=0; i<length; i++)
@@ -275,6 +302,36 @@ wait_for_card_again:
 #endif
 	test_rv(rv, hContext, DONT_PANIC);
 
+	printf("Testing SCardGetAttrib\t\t: ");
+#ifdef USE_AUTOALLOCATE
+	pcbAttrLen = SCARD_AUTOALLOCATE;
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_DEVICE_FRIENDLY_NAME, (unsigned char *)&pbAttr,
+		&pcbAttrLen);
+#else
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_DEVICE_FRIENDLY_NAME, NULL, &pcbAttrLen);
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+	{
+		printf("SCARD_ATTR_DEVICE_FRIENDLY_NAME length: " GREEN "%ld\n" NORMAL, pcbAttrLen);
+		pbAttr = malloc(pcbAttrLen);
+	}
+
+	printf("Testing SCardGetAttrib\t\t: ");
+	rv = SCardGetAttrib(hCard, SCARD_ATTR_DEVICE_FRIENDLY_NAME, pbAttr, &pcbAttrLen);
+#endif
+	test_rv(rv, hContext, DONT_PANIC);
+	if (rv == SCARD_S_SUCCESS)
+		printf("SCARD_ATTR_DEVICE_FRIENDLY_NAME: " GREEN "%s\n" NORMAL, pbAttr);
+
+#ifdef USE_AUTOALLOCATE
+	printf("Testing SCardFreeMemory\t\t: ");
+	rv = SCardFreeMemory(hContext, pbAttr);
+	test_rv(rv, hContext, PANIC);
+#else
+	if (pbAttr)
+		free(pbAttr);
+#endif
+
 	printf("Testing SCardGetAttrib\t\t: ");
 #ifdef USE_AUTOALLOCATE
 	pcbAttrLen = SCARD_AUTOALLOCATE;
